"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const fs = require("fs");
const path = require("path");
const Koa = require("koa");
const bodyparser = require("koa-bodyparser");
const Router = require("koa-router");
const serve = require("koa-static");
const mount = require("koa-mount");
const openapi_backend_1 = require("openapi-backend");
const swagger_ui_dist_1 = require("swagger-ui-dist");
const commonFlags = require("../common/flags");
const koa_1 = require("../common/koa");
class Mock extends command_1.Command {
    async run() {
        const { flags } = this.parse(Mock);
        const { definition, port, 'swagger-ui': swaggerui } = flags;
        const api = new openapi_backend_1.default({ definition });
        api.register({
            validationFail: (c, ctx) => {
                ctx.status = 400;
                ctx.body = { err: c.validation.errors };
            },
            notFound: (c, ctx) => {
                ctx.status = 404;
                ctx.body = { err: 'not found' };
            },
            notImplemented: (c, ctx) => {
                const { status, mock } = c.api.mockResponseForOperation(c.operation.operationId);
                ctx.status = status;
                ctx.body = mock;
            },
        });
        await api.init();
        const app = new Koa();
        app.use(bodyparser());
        // set up cors
        app.use(async (ctx, next) => {
            await next();
            ctx.set('access-control-allow-origin', '*');
        });
        // serve openapi.json
        const openApiFile = 'openapi.json';
        app.use(mount(`/${openApiFile}`, async (ctx, next) => {
            await next();
            const doc = api.document;
            doc.servers = [
                {
                    url: `http://localhost:${port}`,
                },
            ];
            ctx.body = api.document;
            ctx.status = 200;
        }));
        // serve swagger ui
        if (swaggerui) {
            const swaggerUI = new Koa();
            const router = new Router();
            const swaggerUIRoot = swagger_ui_dist_1.getAbsoluteFSPath();
            const indexHTML = fs.readFileSync(path.join(swaggerUIRoot, 'index.html')).toString('utf8');
            router.get('/', (ctx) => {
                if (!ctx.originalUrl.endsWith('/')) {
                    ctx.redirect(`${ctx.originalUrl}/`);
                }
                else {
                    // serve a modified index.html
                    ctx.body = indexHTML
                        // use our openapi definition
                        .replace('https://petstore.swagger.io/v2/swagger.json', `./${openApiFile}`)
                        // display operation ids
                        .replace('layout: "StandaloneLayout"', 'layout: "StandaloneLayout", displayOperationId: true');
                }
            });
            swaggerUI.use(router.routes());
            swaggerUI.use(serve(swaggerUIRoot));
            app.use(mount(`/${swaggerui}`, swaggerUI));
        }
        // serve openapi-backend
        app.use((ctx) => api.handleRequest({
            method: ctx.request.method,
            path: ctx.request.path,
            body: ctx.request.body,
            query: ctx.request.query,
            headers: ctx.request.headers,
        }, ctx));
        // start server
        const { port: portRunning } = await koa_1.startServer({ app, port });
        // print metadata
        this.printInfo(api.document);
        this.printRoutes(api.document);
        this.log(`\nMock server running at http://localhost:${portRunning}`);
        if (swaggerui) {
            this.log(`Swagger UI running at http://localhost:${portRunning}/${swaggerui}`);
        }
        this.log(`OpenAPI definition at http://localhost:${portRunning}/${openApiFile}`);
    }
    printInfo(document) {
        const { title, version, description } = document.info;
        this.log(`\ntitle: ${title}`);
        this.log(`version: ${version}`);
        if (description) {
            this.log(`description: ${description}`);
        }
    }
    printRoutes(document) {
        this.log('\nRoutes:');
        for (const path in document.paths) {
            if (document.paths[path]) {
                for (const method in document.paths[path]) {
                    if (document.paths[path][method]) {
                        const { operationId, summary } = document.paths[path][method];
                        let route = `- ${method.toUpperCase()} ${path}`;
                        if (summary) {
                            route = `${route} - ${summary}`;
                        }
                        if (operationId) {
                            route = `${route} (${operationId})`;
                        }
                        this.log(route);
                    }
                }
            }
        }
    }
}
Mock.description = 'start a local mock API server';
Mock.examples = [
    '$ openapi mock -d ./openapi.yml',
    '$ openapi mock -d https://raw.githubusercontent.com/OAI/OpenAPI-Specification/master/examples/v3.0/petstore.yaml',
];
Mock.flags = Object.assign({}, commonFlags.help(), commonFlags.definition({ required: true }), commonFlags.port({ required: true }), { 'swagger-ui': command_1.flags.string({ char: 'U', description: 'Swagger UI endpoint', helpValue: 'docs' }) });
Mock.args = [];
exports.default = Mock;
