"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const fs = require("fs");
const path = require("path");
const Koa = require("koa");
const Router = require("koa-router");
const serve = require("koa-static");
const SwaggerUIDist = require("swagger-ui-dist");
const commonFlags = require("../common/flags");
const definition_1 = require("../common/definition");
const koa_1 = require("../common/koa");
class SwaggerUI extends command_1.Command {
    async run() {
        const { flags } = this.parse(SwaggerUI);
        const { definition, port, bundle } = flags;
        const app = new Koa();
        const router = new Router();
        let documentPath;
        let document;
        const openApiFile = 'openapi.json';
        if (definition) {
            if (definition.match('://') && !flags.server) {
                // use remote definition
                documentPath = definition;
            }
            else {
                // parse definition
                document = await definition_1.parseDefinition({ definition });
                router.get(`/${openApiFile}`, async (ctx) => {
                    if (flags.server) {
                        const addServers = flags.server.map((url) => ({ url }));
                        document.servers = document.servers ? [...document.servers, ...addServers] : addServers;
                    }
                    ctx.body = document;
                });
                documentPath = `./${openApiFile}`;
            }
        }
        const swaggerUIRoot = SwaggerUIDist.getAbsoluteFSPath();
        // modify index.html
        let indexHTML = fs
            .readFileSync(path.join(swaggerUIRoot, 'index.html'))
            .toString('utf8')
            // display operation ids
            .replace('layout: "StandaloneLayout"', 'layout: "StandaloneLayout", displayOperationId: true');
        if (documentPath) {
            // use our openapi definition
            indexHTML = indexHTML.replace('https://petstore.swagger.io/v2/swagger.json', documentPath);
        }
        // serve from root
        router.get('/', (ctx) => {
            ctx.body = indexHTML;
        });
        app.use(router.routes());
        app.use(serve(swaggerUIRoot));
        if (bundle) {
            // bundle files to directory
            const bundleDir = path.resolve(bundle);
            // create a directory if one does not exist
            if (!fs.existsSync(bundleDir)) {
                fs.mkdirSync(bundleDir);
            }
            // copy dist files
            for (const file of fs.readdirSync(swaggerUIRoot)) {
                const src = path.join(swaggerUIRoot, file);
                const target = path.join(bundleDir, file);
                fs.copyFileSync(src, target);
                this.log(`${target}`);
            }
            if (document) {
                const openApiPath = path.join(bundleDir, openApiFile);
                fs.writeFileSync(openApiPath, JSON.stringify(document));
                this.log(`${openApiPath}`);
            }
            // write index.html
            const indexPath = path.join(bundleDir, 'index.html');
            fs.writeFileSync(indexPath, indexHTML);
            this.log(`${indexPath}`);
        }
        else {
            // run server
            const { port: portRunning } = await koa_1.startServer({ app, port });
            this.log(`Swagger UI running at http://localhost:${portRunning}`);
        }
    }
}
SwaggerUI.description = 'serve or bundle a Swagger UI instance';
SwaggerUI.examples = ['$ openapi swagger-ui', '$ openapi swagger-ui -d ./openapi.yml'];
SwaggerUI.flags = Object.assign({}, commonFlags.help(), commonFlags.definition(), commonFlags.port(), commonFlags.servers(), { bundle: command_1.flags.string({
        char: 'B',
        description: 'bundle a static site to directory',
        helpValue: 'outDir',
    }) });
SwaggerUI.args = [];
exports.default = SwaggerUI;
